// ------ Includes -----

#include "IO.h"
#include "board.h"


static SDL_Surface *mScreen;										// Screen
static Uint32 mColors[COLOR_MAX] = { 0x000000ff,					// Colors
0xff0000ff, 0x00ff00ff, 0x0000ffff,
0x00ffffff, 0xff00ffff, 0xffff00ff,
0xffffffff, 0x4d0974ff };

/*
======================================
Init
======================================
*/
IO::IO()
{
	InitGraph();
}


/*
======================================
Clear the screen to black
======================================
*/
void IO::ClearScreen()
{

	boxColor(mScreen, BOARD_POSITION - (BLOCK_SIZE * (BOARD_WIDTH / 2)) - 1, mScreen->h - (BLOCK_SIZE * BOARD_HEIGHT) - (BLOCK_SIZE * 3), BOARD_POSITION + (BLOCK_SIZE * (BOARD_WIDTH / 2)), mScreen->h -2, mColors[PURPLE]);
	boxColor(mScreen, 480, 240, 480 + (BLOCK_SIZE *5), 240 + (BLOCK_SIZE * 5), mColors[PURPLE]);


	//Barbara added text
	// Load a font
	TTF_Font *font;
	font = TTF_OpenFont("FreeSans.ttf", 14);
	if (font == NULL)
	{
		fprintf(stderr, "TTF_OpenFont() Failed:  %s\n", TTF_GetError());
		TTF_Quit();
		SDL_Quit();
		exit(1);
	}

	// Write text to surface
	SDL_Surface *text;
	SDL_Color text_color = { 255, 255, 255 };
	text = TTF_RenderText_Solid(font,
		"Next Piece:",
		text_color);

	if (text == NULL)
	{
		fprintf(stderr, "TTF_RenderText_Solid() Failed:  %s\n", TTF_GetError());
		TTF_Quit();
		SDL_Quit();
		exit(1);
	}

	//text position
	SDL_Rect dst = { 450, 220 };
	SDL_BlitSurface(text, NULL, mScreen, &dst);
}


/*
======================================
Draw a rectangle of a given color

Parameters:
>> pX1, pY1: 		Upper left corner of the rectangle
>> pX2, pY2: 		Lower right corner of the rectangle
>> pC				Rectangle color
======================================
*/
void IO::DrawRectangle(int pX1, int pY1, int pX2, int pY2, enum color pC)
{
	boxColor(mScreen, pX1, pY1, pX2, pY2 - 1, mColors[pC]);
}


/*
======================================
Return the screen height
======================================
*/
int IO::GetScreenHeight()
{
	return mScreen->h;
}


/*
======================================
Update screen
======================================
*/
void IO::UpdateScreen()
{
	SDL_Flip(mScreen);
}


/*
======================================
Keyboard Input
======================================
*/
int IO::Pollkey()
{
	SDL_Event event;
	while (SDL_PollEvent(&event))
	{
		switch (event.type) {
		case SDL_KEYDOWN:
			return event.key.keysym.sym;
		case SDL_QUIT:
			exit(3);
		}
	}
	return -1;
}

/*
======================================
Keyboard Input
======================================
*/
int IO::Getkey()
{
	SDL_Event event;
	while (true)
	{
		SDL_WaitEvent(&event);
		if (event.type == SDL_KEYDOWN)
			break;
		if (event.type == SDL_QUIT)
			exit(3);
	};
	return event.key.keysym.sym;
}

/*
======================================
Keyboard Input
======================================
*/
int IO::IsKeyDown(int pKey)
{
	Uint8* mKeytable;
	int mNumkeys;
	SDL_PumpEvents();
	mKeytable = SDL_GetKeyState(&mNumkeys);
	return mKeytable[pKey];
}

/*
======================================
SDL Graphical Initialization
======================================
*/
int IO::InitGraph()
{
	const SDL_VideoInfo *info;
	Uint8  video_bpp;
	Uint32 videoflags;

	// Initialize SDL
	if (SDL_Init(SDL_INIT_VIDEO) < 0) {
		fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
		return 1;
	}
	atexit(SDL_Quit);

	// Alpha blending doesn't work well at 8-bit color
	info = SDL_GetVideoInfo();
	if (info->vfmt->BitsPerPixel > 8) {
		video_bpp = info->vfmt->BitsPerPixel;
	}
	else {
		video_bpp = 16;
	}
	videoflags = SDL_SWSURFACE | SDL_DOUBLEBUF;

	// Set 640x480 video mode
	if ((mScreen = SDL_SetVideoMode(640, 480, video_bpp, videoflags)) == NULL) {
		fprintf(stderr, "Couldn't set %ix%i video mode: %s\n", 640, 480, SDL_GetError());
		return 2;
	}
	
	//Barbara changed title of window
	SDL_WM_SetCaption("Tetris", 0);

	//Barbara added background image
	//load bitmap to temp surface 
	SDL_Surface* temp = SDL_LoadBMP("background.bmp");

	//convert bitmap to display format
	SDL_Surface* bg = SDL_DisplayFormat(temp);

	// free the temp surface
	SDL_FreeSurface(temp);

	//draw the background 
	SDL_BlitSurface(bg, NULL, mScreen, NULL);

	//Barbara added text
	// Initialize SDL_ttf library
	if (TTF_Init() != 0)
	{
		fprintf(stderr, "TTF_Init() Failed:  %s\n", TTF_GetError());
		SDL_Quit();
		exit(1);
	}

	return 0;
}